import logging
import re
import shutil
from pathlib import Path

import click
import pkg_resources
import yaml
from kedro import cli


@click.group(name="ARGO")
def commands():
    """ Kedro plugin for transforming kedro pipelines to argo pipelines """
    pass


@commands.command("argo")
@click.argument("image", required=True)
@click.argument("templates_folder", default="templates")
@click.option("--force-template", default=False)
def argokedro(image, templates_folder, force_template):
    """Creates an argo pipeline yaml
    - https://get-ytt.io/#playground
    """
    pc = cli.get_project_context()
    pipeline = pc.pipeline
    dependencies = pipeline.node_dependencies
    deps_dict = get_deps_dict(dependencies)
    tags = get_tags(pipeline)
    tagged_deps_dict = update_deps_dict_with_tags(deps_dict, tags)
    project_name = pc.project_name
    yaml_pipe = generate_yaml(tagged_deps_dict, image, project_name)
    save_yaml(yaml_pipe, templates_folder)
    copy_template(templates_folder, force_template)
    logging.info("Templates saved in `templates` folder")
    click.secho(FINISHED_MESSAGE)


def get_deps_dict(dependencies):
    deps_dict = [
        {
            "node": key.name,
            "name": clean_name(key.name),
            "dep": [clean_name(val.name) for val in vals],
        }
        for key, vals in dependencies.items()
    ]
    return deps_dict


def clean_name(name):
    clean_name = re.sub(r"[\W_]+", "-", name).strip("-")
    return clean_name


def get_tags(pipeline):
    nodes = pipeline.nodes
    tags = {node.name: node.tags for node in nodes}
    return tags


def update_deps_dict_with_tags(deps_dict, tags):
    for dep_dict in deps_dict:
        node_tags = tags[dep_dict["node"]]
        tag_dict = parse_tags(node_tags)
        dep_dict.update(tag_dict)
    return deps_dict


def parse_tags(node_tags, sep="."):
    if node_tags:
        split_tags = [tag.split(sep) for tag in node_tags]
        split_tags = [
            [tag[0], sep.join(tag[1:-1]), tag[-1]]
            for tag in split_tags
            if len(tag) > 2 and tag[0] == "argo"
        ]
        tag_dict = {tag[1]: tag[2] for tag in split_tags}
        return tag_dict
    else:
        return {}


def generate_yaml(deps_dict, image, project_name):
    pipe_dict = {"tasks": deps_dict, "image": image, "project_name": project_name}
    yaml_pipe = yaml.safe_dump(pipe_dict)
    yaml_pipe = "#@data/values\n---\n" + yaml_pipe
    return yaml_pipe


def save_yaml(yaml_pipe, templates_folder):
    Path(templates_folder).mkdir(parents=True, exist_ok=True)
    Path(templates_folder + "/kedro.yaml").write_text(yaml_pipe)


def copy_template(templates_folder, force_template):
    template_filename = "argo_template.yaml"
    source_file = get_source_template_filename(template_filename)
    target_file = Path(templates_folder) / template_filename
    if not target_file.exists() or force_template:
        shutil.copy2(str(source_file), str(templates_folder))


def get_source_template_filename(template_filename="argo_template.yaml"):
    source_file_relative = Path("templates") / template_filename
    source_file = pkg_resources.resource_filename(__name__, str(source_file_relative))
    return source_file


FINISHED_MESSAGE = """
You can now run:

$ ytt -f templates > argo.yaml

or if you prefer in Docker:

$ docker run --rm -it --name ytt -v $(pwd)/templates:/templates gerritk/ytt:latest -f /templates > argo.yaml

and finally

$ argo submit --watch argo.yaml
"""



from pathlib import Path

import pytest
from click.testing import CliRunner
from kedro.pipeline import Pipeline
from kedro.pipeline import node

from kedro_argo.cli import clean_name
from kedro_argo.cli import commands
from kedro_argo.cli import get_deps_dict
from kedro_argo.cli import get_source_template_filename
from kedro_argo.cli import get_tags


def identity(x):
    return x


@pytest.fixture
def mock_pipe():
    mock_node1 = node(identity, "a", "b")
    mock_node2 = node(identity, "b", "c")
    mock_pipe = Pipeline([mock_node1, mock_node2])
    return mock_pipe


@pytest.fixture
def mock_named_pipe():
    mock_node1 = node(identity, "a", "b", name="step_1", tags=["argo.image.what"])
    mock_node2 = node(identity, "b", "c", name="step_2", tags=["argo.huh"])
    mock_named_pipe = Pipeline([mock_node1, mock_node2])
    return mock_named_pipe


def test_main():
    runner = CliRunner()
    result = runner.invoke(commands, [])

    assert result.exit_code == 0


def test_get_deps_dict(mock_named_pipe):
    dependencies = mock_named_pipe.node_dependencies
    deps_dict = get_deps_dict(dependencies)
    expected_deps_dict = [
        {"node": "step_1", "name": "step-1", "dep": []},
        {"node": "step_2", "name": "step-2", "dep": ["step-1"]},
    ]
    assert type(deps_dict) == list
    assert len(dependencies) == len(deps_dict)
    assert deps_dict == expected_deps_dict


def test_get_tags(mock_named_pipe):
    tags = get_tags(mock_named_pipe)
    expected_tags = {"step_1": {"argo.image.what"}, "step_2": {"argo.huh"}}
    assert type(tags) == dict
    assert tags == expected_tags


def test_source_template_filename():
    source_file = get_source_template_filename()
    assert Path(source_file).exists()


def test_clean_name():
    dirty_name = "hi_there[ a ] -> [b]"
    cleaned_name = clean_name(dirty_name)
    expected_name = "hi-there-a-b"
    assert cleaned_name == expected_name



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
from __future__ import absolute_import
from __future__ import print_function

import io
import re
from glob import glob
from os.path import basename
from os.path import dirname
from os.path import join
from os.path import splitext

from setuptools import find_packages
from setuptools import setup


def read(*names, **kwargs):
    with io.open(
        join(dirname(__file__), *names), encoding=kwargs.get("encoding", "utf8")
    ) as fh:
        return fh.read()


setup(
    name="kedro-argo",
    version="0.0.6",
    license="BSD-3-Clause",
    description="Converting kedro pipelines to argo pipelines.",
    long_description="%s\n%s"
    % (
        re.compile("^.. start-badges.*^.. end-badges", re.M | re.S).sub(
            "", read("README.rst")
        ),
        re.sub(":[a-z]+:`~?(.*?)`", r"``\1``", read("CHANGELOG.rst")),
    ),
    author="Andrej Marsic",
    author_email="marsic.andrej@gmail.com",
    url="https://github.com/nraw/kedro-argo",
    packages=find_packages("src"),
    package_dir={"": "src"},
    py_modules=[splitext(basename(path))[0] for path in glob("src/*.py")],
    include_package_data=True,
    zip_safe=False,
    classifiers=[
        # complete classifier list: http://pypi.python.org/pypi?%3Aaction=list_classifiers
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: BSD License",
        "Operating System :: Unix",
        "Operating System :: POSIX",
        "Operating System :: Microsoft :: Windows",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: Implementation :: CPython",
        # uncomment if you test on these interpreters:
        # 'Programming Language :: Python :: Implementation :: IronPython',
        # 'Programming Language :: Python :: Implementation :: Jython',
        # 'Programming Language :: Python :: Implementation :: Stackless',
        "Topic :: Utilities",
    ],
    project_urls={
        "Documentation": "https://kedro-argo.readthedocs.io/",
        "Changelog": "https://kedro-argo.readthedocs.io/en/latest/changelog.html",
        "Issue Tracker": "https://github.com/nraw/kedro-argo/issues",
    },
    keywords=[
        # eg: 'keyword1', 'keyword2', 'keyword3',
    ],
    python_requires=">=3.6, <3.8",
    install_requires=[
        "click>=7.0.0",
        "kedro>=0.15.5",
        # eg: 'aspectlib==1.1.1', 'six>=1.7',
    ],
    extras_require={
        # eg:
        #   'rst': ['docutils>=0.11'],
        #   ':python_version=="2.6"': ['argparse'],
    },
    entry_points={
        "console_scripts": ["kedro-argo = kedro_argo.cli:commands"],
        "kedro.project_commands": ["argo = kedro_argo.cli:commands"],
    },
)
